<?php namespace Flarum\Core\Handlers\Commands;

use Flarum\Core\Repositories\DiscussionRepositoryInterface as DiscussionRepository;
use Flarum\Core\Events\DiscussionWillBeSaved;
use Flarum\Core\Support\DispatchesEvents;

class EditDiscussionCommandHandler
{
    use DispatchesEvents;

    protected $discussions;

    public function __construct(DiscussionRepository $discussions)
    {
        $this->discussions = $discussions;
    }

    public function handle($command)
    {
        $user = $command->user;
        $discussion = $this->discussions->findOrFail($command->discussionId, $user);

        if (isset($command->data['title'])) {
            $discussion->assertCan($user, 'rename');
            $discussion->rename($command->data['title'], $user);
        }

        event(new DiscussionWillBeSaved($discussion, $command));

        $discussion->save();
        $this->dispatchEventsFor($discussion);

        return $discussion;
    }
}
