<?php namespace Flarum\Core\Handlers\Commands;

use Flarum\Core\Repositories\PostRepositoryInterface as PostRepository;
use Flarum\Core\Events\PostWillBeSaved;
use Flarum\Core\Support\DispatchesEvents;
use Flarum\Core\Models\CommentPost;

class EditPostCommandHandler
{
    use DispatchesEvents;

    protected $posts;

    public function __construct(PostRepository $posts)
    {
        $this->posts = $posts;
    }

    public function handle($command)
    {
        $user = $command->user;
        $post = $this->posts->findOrFail($command->postId, $user);

        if ($post instanceof CommentPost) {
            if (isset($command->data['content'])) {
                $post->assertCan($user, 'edit');

                $post->revise($command->data['content'], $user);
            }

            if (isset($command->data['isHidden'])) {
                $post->assertCan($user, 'edit');

                if ($command->data['isHidden']) {
                    $post->hide($user);
                } else {
                    $post->restore($user);
                }
            }
        }

        event(new PostWillBeSaved($post, $command));

        $post->save();
        $this->dispatchEventsFor($post);

        return $post;
    }
}
